@@ -479,6 +479,20 @@ cs_activate_usb_storage (GtkWidget *widget, gpointer data)
 
     if (!options->usb_storage && USB_FLAG) {
 
+        gchar *active_env = nm_environment_manager_get_active();
+        GtkWidget *usb = nm_usb_setup_new(active_env);
+        nm_usb_setup_enable(NM_USB_SETUP(usb), NM_USB_MASS_STORAGE_LABEL);
+        gint ret = nm_usb_setup_apply_changes(NM_CONN_WIDGET(usb));
+        
+        g_free(usb);
+        g_free(active_env);
+        
+        if (!ret) {
+            hildon_banner_show_information(NULL, NULL, _("Error enabling USB file storage"));
+            return;
+        }
+        
+        
         USB_FLAG = !USB_FLAG;
         g_object_set_property (G_OBJECT (options->menu_usb_storage),
                                 "active", &vt);
@@ -486,8 +500,6 @@ cs_activate_usb_storage (GtkWidget *widget, gpointer data)
                                 "active", &vf);
         USB_FLAG = !USB_FLAG;
 
-        system("sudo /usr/sbin/nm-usb-setup stop");
-
         options->usb_netdev = FALSE;
         options->usb_storage = TRUE;
 
@@ -509,30 +521,37 @@ cs_activate_usb_netdev (GtkWidget *widget, gpointer data)
 
     if (!options->usb_netdev && USB_FLAG) {
 
-        gchar *gw;
-        NmSettings *settings = nm_settings_new ();
-
-        if (!nm_settings_start (settings)) {
-
-            hildon_banner_show_information(NULL, NULL,
-                                            _("Please, open PC-Connectivity manager and configure it."));
+        gchar *active_env = nm_environment_manager_get_active();
+        GtkWidget *usb = nm_usb_setup_new(active_env);
+        nm_usb_setup_enable(NM_USB_SETUP(usb), NM_USB_NETWORK_LABEL);
+        gint ret = nm_usb_setup_apply_changes(NM_CONN_WIDGET(usb));
+        
+        g_free(usb);
+        g_free(active_env);
+        
+        if (!ret) {
+            hildon_banner_show_information(NULL, NULL, _("Error enabling USB network"));
             return;
         }
 
-        gw = nm_settings_get_value (settings, nm_environment_manager_get_active(), NM_USB_SETTINGS_GROUP,
-                                     NM_SETTINGS_GATEWAY);
+/*        gchar *gw;*/
+/*        NmSettings *settings = nm_settings_new ();*/
+
+/*        if (!nm_settings_start (settings)) {*/
+
+/*            hildon_banner_show_information(NULL, NULL,*/
+/*                                            _("Please, open PC-Connectivity manager and configure it."));*/
+/*            return;*/
+/*        }*/
+
+/*        gw = nm_settings_get_value (settings, nm_environment_manager_get_active(), NM_USB_SETTINGS_GROUP,*/
+/*                                     NM_SETTINGS_GATEWAY);*/
 
         USB_FLAG = !USB_FLAG;
         g_object_set_property (G_OBJECT (options->menu_usb_storage), "active", &vf);
         g_object_set_property (G_OBJECT (options->menu_usb_netdev), "active", &vt);
         USB_FLAG = !USB_FLAG;
 
-        system("sudo /usr/sbin/nm-usb-setup start");
-        system (g_strconcat ("sudo /sbin/ip route del default via ", gw,
-                              " dev usb0", NULL));
-        system (g_strconcat ("sudo /sbin/ip route append default via ", gw,
-                              " dev usb0", NULL));
-
         options->usb_netdev = TRUE;
         options->usb_storage = FALSE;
 
@@ -554,42 +573,19 @@ cs_activate_bt_netdev (GtkWidget *widget, gpointer data)
 
     if (!options->bt_netdev && BT_FLAG) {
 
-        gchar *btaddr, *gw;
-        NmSettings *settings = nm_settings_new ();
-
-        if (!nm_settings_start (settings)) {
-
-            hildon_banner_show_information(NULL, NULL,
-                                            _("Please, open PC-Connectivity manager and configure it."));
+        gchar *active_env = nm_environment_manager_get_active();
+        GtkWidget *bt = nm_bt_setup_new(active_env);
+        nm_bt_setup_enable(NM_BT_SETUP(bt), TRUE);
+        gint ret = nm_bt_setup_apply_changes(NM_CONN_WIDGET(bt));
+        
+        g_free(bt);
+        g_free(active_env);
+        
+        if (!ret) {
+            hildon_banner_show_information(NULL, NULL, _("Error enabling BT network"));
             return;
         }
 
-        btaddr = nm_settings_get_value (settings, nm_environment_manager_get_active(), NM_BT_SETTINGS_GROUP,
-                                         NM_SETTINGS_MAC);
-        gw = nm_settings_get_value (settings, nm_environment_manager_get_active(), NM_BT_SETTINGS_GROUP,
-                                     NM_SETTINGS_GATEWAY);
-
-        if (strcmp (btaddr, NM_BT_DEFAULT_MAC) == 0) {
-
-            hildon_banner_show_information (NULL, NULL,
-                                             _("Please, open network manager and\nconfigure bluetooth interface!"));
-            return;
-        } else if (nm_bt_utils_check_pand (options->bt_utils, btaddr) == NULL) {
-
-            hildon_banner_show_information (NULL, NULL,
-                                            _("Unable to connect. Check if PAND is running on host!"));
-            return;
-        }
-
-        if (!nm_bt_utils_connect (options->bt_utils, btaddr, gw)) {
-
-            hildon_banner_show_information (NULL, NULL,
-                                            _("Could not connect to PC"));
-            return;
-        }
-
-        hildon_banner_show_information(NULL, NULL, _("Bluetooth network enabled"));
-
         BT_FLAG = !BT_FLAG;
         g_object_set_property (G_OBJECT (options->menu_bt_netdev),
                                 "active", &vt);
@@ -597,7 +593,18 @@ cs_activate_bt_netdev (GtkWidget *widget, gpointer data)
         options->bt_netdev = TRUE;
     } else if (BT_FLAG) {
 
-        nm_bt_utils_remove_connection (options->bt_utils);
+        gchar *active_env = nm_environment_manager_get_active();
+        GtkWidget *bt = nm_bt_setup_new(active_env);
+        nm_bt_setup_enable(NM_BT_SETUP(bt), FALSE);
+        gint ret = nm_bt_setup_apply_changes(NM_CONN_WIDGET(bt));
+        
+        g_free(bt);
+        g_free(active_env);
+        
+        if (!ret) {
+            hildon_banner_show_information(NULL, NULL, _("Error disabling BT network"));
+            return;
+        }
 
         hildon_banner_show_information(NULL, NULL,
                                         _("Bluetooth network disabled"));
@@ -626,10 +633,20 @@ cs_activate_wlan_netdev (GtkWidget *widget, gpointer data)
 
     if (!options->wlan_netdev && WLAN_FLAG) {
 
-        status = system ("dbus-send --system --dest=com.nokia.icd /com/nokia/icd_ui com.nokia.icd_ui.disconnect boolean:true");
-        g_usleep(2 * 1000000);
-        status = system ("dbus-send --type=method_call --system --dest=com.nokia.icd /com/nokia/icd com.nokia.icd.connect string:devel_adhoc uint32:0");
-        g_usleep(2 * 1000000);
+        gchar *active_env = nm_environment_manager_get_active();
+        GtkWidget *wlan = nm_wlan_setup_new(active_env);
+        nm_wlan_setup_enable(NM_WLAN_SETUP(wlan), TRUE);
+        gint ret = nm_wlan_setup_apply_changes(NM_CONN_WIDGET(wlan));
+        
+        g_free(wlan);
+        g_free(active_env);
+        
+        if (!ret) {
+            hildon_banner_show_information(NULL, NULL, _("Error enabling Wlan network"));
+            return;
+        }
+
+
         WLAN_FLAG = !WLAN_FLAG;
         g_object_set_property (G_OBJECT (options->menu_wlan_netdev), "active",
                                 &vt);
@@ -639,8 +656,19 @@ cs_activate_wlan_netdev (GtkWidget *widget, gpointer data)
         hildon_banner_show_information(NULL, NULL, _("WLAN adhoc network enabled"));
     } else if (options->wlan_netdev && WLAN_FLAG) {
 
-        // FIXME: reconnect to previous wlan network
-        status = system ("dbus-send --system --dest=com.nokia.icd /com/nokia/icd_ui com.nokia.icd_ui.disconnect boolean:true");
+        gchar *active_env = nm_environment_manager_get_active();
+        GtkWidget *wlan = nm_wlan_setup_new(active_env);
+        nm_wlan_setup_enable(NM_WLAN_SETUP(wlan), FALSE);
+        gint ret = nm_wlan_setup_apply_changes(NM_CONN_WIDGET(wlan));
+        
+        g_free(wlan);
+        g_free(active_env);
+        
+        if (!ret) {
+            hildon_banner_show_information(NULL, NULL, _("Error disabling Wlan network"));
+            return;
+        }
+
         WLAN_FLAG = !WLAN_FLAG;
         g_object_set_property (G_OBJECT (options->menu_wlan_netdev), "active",
                                 &vf);
